#include <iostream>
#include <KernelMPI.h>

#include "benchmark_utils.h"
#include "benchmark_DrawElements.h"

#include "Fixed.h"

LF_USING_BRIO_NAMESPACE()


void TestDrawElements(int iBillboards, const int iBillboardSize=32, const int iTextureSize=512)
{
	char sTestName[] = "DrawElements";
	//print_test_start(sTestName);
	CKernelMPI kernel;
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	const int iCoordsPerVertex = COORDS_PER_VERTEX;			// 2, 3, or 4
	const int iCoordsPerTextureVertex = COORDS_PER_TEXTURE_VERTEX;	// 2, 3, or 4
	const tZMode zMode = USE_ZMODE;
	const int iVerticesPerBillboard = 4;		// 4 if indexed, 6 otherwise
	
	// handle textures
	GLuint* aTextureID = new GLuint[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureID[i] = createTexture(iTextureSize, USE_PIXEL_FORMAT);
	}
	
	// handle vertex array
	GLfixed** aVertexArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aVertexArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerVertex];
		createIndexedVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
	}
	GLuint* aVertexBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aVertexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
		glBufferData(GL_ARRAY_BUFFER, 
				iBillboards * 4 * iCoordsPerVertex * sizeof(GLfixed), 
				aVertexArray[i], 
				GL_DYNAMIC_DRAW);
		glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, NULL);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
	
	// handle texture coordinates
	GLfixed** aTextureCoordArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureCoordArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerTextureVertex];
		createIndexedTextureCoordArrayx(aTextureCoordArray[i], iBillboards, iBillboardSize, iTextureSize, iCoordsPerVertex);
	}
	GLuint* aTextureCoordBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aTextureCoordBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ARRAY_BUFFER, aTextureCoordBufferID[i]);
		glBufferData(GL_ARRAY_BUFFER, 
				iBillboards * 4 * iCoordsPerVertex * sizeof(GLfixed), 
				aTextureCoordArray[i], 
				GL_DYNAMIC_DRAW);
		glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, 0, NULL);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
	
	// handle index array
	int iTotalIndices = iBillboards * 6;
	GLushort** aIndexArray = new GLushort*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aIndexArray[i] = new GLushort[iTotalIndices];
		createIndexArray(aIndexArray[i], iBillboards);
	}
//	GLuint* aIndexBufferID = new GLuint[NUM_TEXTURE_SHEETS];
//	glGenBuffers(NUM_TEXTURE_SHEETS, aIndexBufferID);
//	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
//	{
//		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, aIndexBufferID[i]);
//		glBufferData(GL_ELEMENT_ARRAY_BUFFER, iTotalIndices * sizeof(GLushort), aIndexArray[i], GL_STATIC_DRAW);
//		// unbind the VBO to reuse in the future
//		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
//	}
	
	// draw
	int frames = NUMBER_FRAMES;
	U64 startTime = kernel.GetElapsedTimeAsMSecs();
	while(frames--)
	{
		gl_clearBuffer();
		
		for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
		{
			if (NUM_TEXTURE_SHEETS>1)
			{
				// swap textures
				glBindTexture(GL_TEXTURE_2D, aTextureID[i]);
			}
			if(REWRITE_VERTEX_ARRAY_EACH_FRAME)
			{
				createIndexedVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
			}
			//glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, aVertexArray[i]);
			//glTexCoordPointer(iCoordsPerVertex, GL_FIXED, 0, aTextureCoordArray[i]);
			//glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, aIndexArray[i]);
			
			glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
			glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, NULL);
			glBindBuffer(GL_ARRAY_BUFFER, aTextureCoordBufferID[i]);
			glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, 0, NULL);
//			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, aIndexBufferID[i]);
//			glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, NULL);
			glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, aIndexArray[i]);
		}
		
		gl_swapBuffer();
	}
	U64 endTime = kernel.GetElapsedTimeAsMSecs();
	U64 renderTime = endTime - startTime;
	print_test_results(sTestName, iBillboardSize, iBillboards*NUM_TEXTURE_SHEETS, (double)NUMBER_FRAMES * 1000 / renderTime);

	// cleanup
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	// vertex buffer unbind
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glDeleteTextures(NUM_TEXTURE_SHEETS, aTextureID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aVertexBufferID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aTextureCoordBufferID);
//	glDeleteBuffers(NUM_TEXTURE_SHEETS, aIndexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		delete[] aVertexArray[i];
		delete[] aTextureCoordArray[i];
		delete[] aIndexArray[i];
	}
	delete[] aTextureID;
	delete[] aVertexBufferID;
	delete[] aTextureCoordBufferID;
//	delete[] aIndexBufferID;
	delete[] aVertexArray;
	delete[] aTextureCoordArray;
	delete[] aIndexArray;
}


void TestDrawArrays(int iBillboards, const int iBillboardSize=32, const int iTextureSize=512)
{
	char sTestName[] = "DrawArrays";
	//print_test_start(sTestName);
	CKernelMPI kernel;
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	const int iCoordsPerVertex = COORDS_PER_VERTEX;			// 2, 3, or 4
	const int iCoordsPerTextureVertex = COORDS_PER_TEXTURE_VERTEX;	// 2, 3, or 4
	const tZMode zMode = USE_ZMODE;
	const int iVerticesPerBillboard = 6;		// 4 if indexed, 6 otherwise
	
	// handle textures
	GLuint* aTextureID = new GLuint[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureID[i] = createTexture(iTextureSize, USE_PIXEL_FORMAT);
	}
	
	// handle vertex array
	GLfixed** aVertexArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aVertexArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerVertex];
		createVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
	}
	
	// handle texture coordinates
	GLfixed** aTextureCoordArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureCoordArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerTextureVertex];
		createTextureCoordArrayx(aTextureCoordArray[i], iBillboards, iBillboardSize, iTextureSize, iCoordsPerVertex);
	}
	
	// draw
	int frames = NUMBER_FRAMES;
	U64 startTime = kernel.GetElapsedTimeAsMSecs();
	while(frames--)
	{
		gl_clearBuffer();
		
		for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
		{
			if (NUM_TEXTURE_SHEETS>1)
			{
				// swap textures
				glBindTexture(GL_TEXTURE_2D, aTextureID[i]);
			}
			if(REWRITE_VERTEX_ARRAY_EACH_FRAME)
			{
				createVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
			}
			glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, aVertexArray[i]);
			glTexCoordPointer(iCoordsPerVertex, GL_FIXED, 0, aTextureCoordArray[i]);
			glDrawArrays(GL_TRIANGLES, 0, iBillboards*iVerticesPerBillboard);
		}
		gl_swapBuffer();
	}
	U64 endTime = kernel.GetElapsedTimeAsMSecs();
	U64 renderTime = endTime - startTime;
	print_test_results(sTestName, iBillboardSize, iBillboards*NUM_TEXTURE_SHEETS, (double)NUMBER_FRAMES * 1000 / renderTime);

	// cleanup
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	// vertex buffer unbind
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glDeleteTextures(NUM_TEXTURE_SHEETS, aTextureID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		delete[] aVertexArray[i];
		delete[] aTextureCoordArray[i];
	}
	delete[] aTextureID;
	delete[] aVertexArray;
	delete[] aTextureCoordArray;
}



void test_DrawElements(int maxBillboards, int stride)
{
	gl_disableAll();
	gl_enableDepthBuffer();
	gl_enableBlending();
	gl_enableAlpha();
	glEnable(GL_TEXTURE_2D);
	if(gl_checkError())
	{
		printf("Error:  An error occured in test_DrawElements after disabling and enabling features.\n");
	}
	
	for(int i = stride; i <= maxBillboards; i += stride)
	{
		TestDrawElements(i, 32);
		TestDrawElements(i, 20);
		TestDrawElements(i, 16);
		TestDrawElements(i, 10);
		TestDrawElements(i, 8);
		
//		TestDrawArrays(i, 32);
//		TestDrawArrays(i, 20);
//		TestDrawArrays(i, 16);
//		TestDrawArrays(i, 10);
//		TestDrawArrays(i, 8);
		
		if(gl_checkError())
		{
			printf("Error:  An error occured in test_DrawElements after tests were run.\n");
		}
	}
}
