#define	lf_linux
#define	__int64	long long

#include <iostream>
//#include <KernelMPI.h>

#include "benchmark_utils.h"
#include "benchmark_VertexBuffer.h"

//#include "Fixed.h"
#include "render.h"

#define tFixed(x)		FixedFromFloat(x)
#define ONE_X			ONE
#define U8				unsigned char
#define U16				unsigned short
#define U32				unsigned long	

extern U32	GetElapsedTimeAsMSecs();

#define SUPPORT_LIB     1

//LF_USING_BRIO_NAMESPACE()

// Test the LF1000 standard OpenGL technique using Vertex Buffer Objects
// This does not use indexed arrays
void TestVertexBuffer(int iBillboards, const int iBillboardSize=32, const int iTextureSize=128)
{
	char sTestName[] = "VertexBuffer";
	//print_test_start(sTestName);
	//CKernelMPI kernel;
#if SUPPORT_LIB

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	const int iCoordsPerVertex = COORDS_PER_VERTEX;			// 2, 3, or 4
	const int iCoordsPerTextureVertex = COORDS_PER_TEXTURE_VERTEX;	// 2, 3, or 4
	const tZMode zMode = USE_ZMODE;
	const int iVerticesPerBillboard = 6;		// 4 if indexed, 6 otherwise
	
	// handle textures
	GLuint* aTextureID = new GLuint[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureID[i] = createTexture(iTextureSize, USE_PIXEL_FORMAT);
	}
	
	// handle vertex array
	GLfixed** aVertexArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aVertexArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerVertex];
		createVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
	}
	GLuint* aVertexBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aVertexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
		glBufferData(GL_ARRAY_BUFFER, 
				iBillboards * iVerticesPerBillboard * iCoordsPerVertex * sizeof(GLfixed), 
				aVertexArray[i], 
				GL_DYNAMIC_DRAW);
		glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, NULL);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
	
	// handle texture coordinates
	GLfixed** aTextureCoordArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureCoordArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerTextureVertex];
		createTextureCoordArrayx(aTextureCoordArray[i], iBillboards, iBillboardSize, iTextureSize, iCoordsPerVertex);
	}
	GLuint* aTextureCoordBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aTextureCoordBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ARRAY_BUFFER, aTextureCoordBufferID[i]);
		glBufferData(GL_ARRAY_BUFFER, 
				iBillboards * iVerticesPerBillboard * iCoordsPerVertex * sizeof(GLfixed), 
				aTextureCoordArray[i], 
				GL_DYNAMIC_DRAW);
		glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, 0, NULL);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
	
	// handle index array
//	int iTotalIndices = iBillboards * 6;
//	GLushort** aIndexArray = new GLushort*[NUM_TEXTURE_SHEETS];
//	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
//	{
//		aIndexArray[i] = new GLushort[iTotalIndices];
//		createIndexArray(aIndexArray[i], iBillboards);
//	}
//	GLuint* aIndexBufferID = new GLuint[NUM_TEXTURE_SHEETS];
//	glGenBuffers(NUM_TEXTURE_SHEETS, aIndexBufferID);
//	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
//	{
//		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, aIndexBufferID[i]);
//		glBufferData(GL_ELEMENT_ARRAY_BUFFER, iTotalIndices * sizeof(GLushort), aIndexArray[i], GL_STATIC_DRAW);
//		// unbind the VBO to reuse in the future
//		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
//	}
	
	
	// draw
	int frames = NUMBER_FRAMES;
	U32 startTime = GetElapsedTimeAsMSecs();
	while(frames--)
	{
		gl_clearBuffer();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrthox(0, tFixed(320),0, tFixed(240),tFixed(-10), tFixed(1));
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		
		for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
		{
		      glLoadIdentity();
		      glTranslatex(0, 0, 0);
		      glRotatex(0, 0, 0, 65536);
		/*
		      glScalex(65536, 65536, 65536);
		*/
			
			if (NUM_TEXTURE_SHEETS>1)
			{
				//printf("swapping textures to %d\n", aTextureID[i]);
				// swap textures
				glBindTexture(GL_TEXTURE_2D, aTextureID[i]);
			}
			if(REWRITE_VERTEX_ARRAY_EACH_FRAME)
			{
				createVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
				glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
				glBufferData(GL_ARRAY_BUFFER, 
						iBillboards * iVerticesPerBillboard * iCoordsPerVertex * sizeof(GLfixed), 
						aVertexArray[i], 
						GL_DYNAMIC_DRAW);
			}
			else
			{
				glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
			}
			glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, NULL);
			glBindBuffer(GL_ARRAY_BUFFER, aTextureCoordBufferID[i]);
			glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, 0, NULL);
			//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, aIndexBufferID[i]);
			//glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, NULL);
			glDrawArrays(GL_TRIANGLES, 0, iBillboards*iVerticesPerBillboard);
		}
		
		gl_swapBuffer();
	}
	U32 endTime = GetElapsedTimeAsMSecs();
	U32 renderTime = endTime - startTime;
	print_test_results(sTestName, iBillboardSize, iBillboards*NUM_TEXTURE_SHEETS, (double)NUMBER_FRAMES * 1000 / renderTime);
	
	// cleanup
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	// vertex buffer unbind
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glDeleteTextures(NUM_TEXTURE_SHEETS, aTextureID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aVertexBufferID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aTextureCoordBufferID);
	//glDeleteBuffers(NUM_TEXTURE_SHEETS, aIndexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		delete[] aVertexArray[i];
		delete[] aTextureCoordArray[i];
		//delete[] aIndexArray[i];
	}
	delete[] aTextureID;
	delete[] aVertexBufferID;
	delete[] aTextureCoordBufferID;
	//delete[] aIndexBufferID;
	delete[] aVertexArray;
	delete[] aTextureCoordArray;
	//delete[] aIndexArray;

#endif

}


// Test the LF1000 standard OpenGL technique using Vertex Buffer Objects using and Index array
void TestVertexBufferIndexed(int iBillboards, const int iBillboardSize=32, const int iTextureSize=128)
{
	char sTestName[] = "VertexBufferIndexed";

#if SUPPORT_LIB

	//print_test_start(sTestName);
	//CKernelMPI kernel;
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	const int iCoordsPerVertex = COORDS_PER_VERTEX;			// 2, 3, or 4
	const int iCoordsPerTextureVertex = COORDS_PER_TEXTURE_VERTEX;	// 2, 3, or 4
	const tZMode zMode = USE_ZMODE;
	const int iVerticesPerBillboard = 6;		// 4 if indexed, 6 otherwise
	
	// handle textures
	GLuint* aTextureID = new GLuint[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureID[i] = createTexture(iTextureSize, USE_PIXEL_FORMAT);
	}
	
	// handle vertex array
	GLfixed** aVertexArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aVertexArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerVertex];
		createIndexedVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
	}
	GLuint* aVertexBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aVertexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
		glBufferData(GL_ARRAY_BUFFER, 
				iBillboards * iVerticesPerBillboard * iCoordsPerVertex * sizeof(GLfixed), 
				aVertexArray[i], 
				GL_DYNAMIC_DRAW);
		glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, NULL);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
	
	// handle texture coordinates
	GLfixed** aTextureCoordArray = new GLfixed*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aTextureCoordArray[i] = new GLfixed[iBillboards * iVerticesPerBillboard * iCoordsPerTextureVertex];
		createIndexedTextureCoordArrayx(aTextureCoordArray[i], iBillboards, iBillboardSize, iTextureSize, iCoordsPerVertex);
	}
	GLuint* aTextureCoordBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aTextureCoordBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ARRAY_BUFFER, aTextureCoordBufferID[i]);
		glBufferData(GL_ARRAY_BUFFER, 
				iBillboards * iVerticesPerBillboard * iCoordsPerVertex * sizeof(GLfixed), 
				aTextureCoordArray[i], 
				GL_DYNAMIC_DRAW);
		glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, 0, NULL);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}
	
	// handle index array
	int iTotalIndices = iBillboards * 6;
	GLushort** aIndexArray = new GLushort*[NUM_TEXTURE_SHEETS];
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		aIndexArray[i] = new GLushort[iTotalIndices];
		createIndexArray(aIndexArray[i], iBillboards);
	}
	GLuint* aIndexBufferID = new GLuint[NUM_TEXTURE_SHEETS];
	glGenBuffers(NUM_TEXTURE_SHEETS, aIndexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, aIndexBufferID[i]);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, iTotalIndices * sizeof(GLushort), aIndexArray[i], GL_STATIC_DRAW);
		// unbind the VBO to reuse in the future
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	}
	
	// draw
	int frames = NUMBER_FRAMES;
	//U64 startTime = kernel.GetElapsedTimeAsMSecs();
	while(frames--)
	{
		gl_clearBuffer();
		
		for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
		{
			if (NUM_TEXTURE_SHEETS>1)
			{
				// swap textures
				glBindTexture(GL_TEXTURE_2D, aTextureID[i]);
			}
			if(REWRITE_VERTEX_ARRAY_EACH_FRAME)
			{
				createVertexArrayx(aVertexArray[i], iBillboards, iBillboardSize, iCoordsPerVertex, zMode);
				glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
				glBufferData(GL_ARRAY_BUFFER, 
						iBillboards * iVerticesPerBillboard * iCoordsPerVertex * sizeof(GLfixed), 
						aVertexArray[i], 
						GL_DYNAMIC_DRAW);
			}
			else
			{
				glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferID[i]);
			}
			glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, NULL);
			glBindBuffer(GL_ARRAY_BUFFER, aTextureCoordBufferID[i]);
			glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, 0, NULL);
			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, aIndexBufferID[i]);
			glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, NULL);
		}
		
		/*
		// sanity check - try drawing without the VBO
		for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
		{
			glVertexPointer(iCoordsPerVertex, GL_FIXED, 0, aVertexArray[i]);
			glTexCoordPointer(iCoordsPerVertex, GL_FIXED, 0, aTextureCoordArray[i]);
			glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, indexArray[i]);
		}
		*/
		gl_swapBuffer();
	}
	//U64 endTime = kernel.GetElapsedTimeAsMSecs();
	//U64 renderTime = endTime - startTime;
	//print_test_results(sTestName, iBillboardSize, iBillboards*NUM_TEXTURE_SHEETS, (double)NUMBER_FRAMES * 1000 / renderTime);

	// cleanup
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	// vertex buffer unbind
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glDeleteTextures(NUM_TEXTURE_SHEETS, aTextureID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aVertexBufferID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aTextureCoordBufferID);
	glDeleteBuffers(NUM_TEXTURE_SHEETS, aIndexBufferID);
	for (int i=0; i<NUM_TEXTURE_SHEETS; ++i)
	{
		delete[] aVertexArray[i];
		delete[] aTextureCoordArray[i];
		delete[] aIndexArray[i];
	}
	delete[] aTextureID;
	delete[] aVertexBufferID;
	delete[] aTextureCoordBufferID;
	delete[] aIndexBufferID;
	delete[] aVertexArray;
	delete[] aTextureCoordArray;
	delete[] aIndexArray;

#endif
}

//void TestSingleVertexBuffer(int iBillboards, const int iBillboardSize=32, const int iTextureSize=512)
//{
//	glEnableClientState(GL_VERTEX_ARRAY);
//	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
//	CKernelMPI kernel;
//	
//	GLuint textureID = createTexture(iTextureSize, USE_PIXEL_FORMAT);
//
//	const int iCoordsPerVertex = 4;
//	const int iCoordsPerTextureVertex = 4;
//	const tZMode zMode = USE_ZMODE;
//	
//	// build a single buffer for both vertices and texture coordinates
//	GLfixed* vertexTextureArray = new GLfixed[iBillboards * 6 * (iCoordsPerVertex+iCoordsPerTextureVertex)];
//	createVertexTextureArrayx(vertexTextureArray, iBillboards, iBillboardSize, iTextureSize, iCoordsPerVertex, iCoordsPerTextureVertex, zMode);
//	
//	// handle the vertex-texture array
//	GLuint vertexTextureBufferID;
//	glGenBuffers(1, &vertexTextureBufferID);
//	glBindBuffer(GL_ARRAY_BUFFER, vertexTextureBufferID);
//	glBufferData(GL_ARRAY_BUFFER, iBillboards * 6 * (iCoordsPerVertex+iCoordsPerTextureVertex) * sizeof(GLfixed), vertexTextureArray, GL_DYNAMIC_DRAW);
//	U32 vertexStride = iCoordsPerTextureVertex * sizeof(GLfixed);	// jump over the texture coord data
//	glVertexPointer(iCoordsPerVertex, GL_FIXED, vertexStride, NULL);
//
//	U32 texCoordStride = iCoordsPerVertex * sizeof(GLfixed);	// jump over the texture coord data
//	glTexCoordPointer(iCoordsPerTextureVertex, GL_FIXED, texCoordStride, NULL);
//	
//	/*
//	// handle index array
//	int iTotalIndices = iBillboards * 6;
//	GLushort *indexArray0 = new GLushort[iTotalIndices];
//	createIndexArray(indexArray0, iBillboards);
//	
//	GLuint indexBufferID0;
//	glGenBuffers(1, &indexBufferID0);
//	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBufferID0);
//	glBufferData(GL_ELEMENT_ARRAY_BUFFER, iTotalIndices * sizeof(GLushort), indexArray0, GL_STATIC_DRAW);
//	// unbind the VBO to reuse in the future
//	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
//	*/
//	
//	// draw
//	//glEnableClientState(GL_VERTEX_ARRAY);
//	//glEnableClientState(GL_TEXTURE_COORD_ARRAY);
//	int frames = NUMBER_FRAMES;
//	U64 startTime = kernel.GetElapsedTimeAsMSecs();
//	while(frames--)
//	{
//		gl_clearBuffer();
//		
//		glBindBuffer(GL_ARRAY_BUFFER, vertexTextureBufferID);
//		glVertexPointer(2, GL_FIXED, vertexStride, NULL);
//		//glBindBuffer(GL_ARRAY_BUFFER, textureCoordBufferID0);
//		glTexCoordPointer(2, GL_FIXED, texCoordStride, NULL);
//		
//		//glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBufferID0);
//		//glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, NULL);
//		glDrawArrays(GL_TRIANGLES, 0, iBillboards * 6);
//		/*
//		// try drawing without the VBO
//		glVertexPointer(2, GL_FIXED, 0, vertexArray);
//		glTexCoordPointer(2, GL_FIXED, 0, textureArray);
//		glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, indexArray0);
//		*/
//		gl_swapBuffer();
//	}
//	U64 endTime = kernel.GetElapsedTimeAsMSecs();
//	U64 renderTime = endTime - startTime;
//	printf("SingleVertexBuffer size=%d billboards=%d fps=%f\n", iBillboardSize, iBillboards, (double)NUMBER_FRAMES * 1000 / renderTime);
//	// cleanup
//	glDisableClientState(GL_VERTEX_ARRAY);
//	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
//	// vertex buffer unbind
//	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
//	glBindBuffer(GL_ARRAY_BUFFER, 0);
//	delete[] vertexTextureArray;
//	//delete[] indexArray0;
//	//glDeleteBuffers(1, &vertexBufferID0);
//	//glDeleteBuffers(1, &textureCoordBufferID0);
//	glDeleteBuffers(1, &vertexTextureBufferID);
//	//glDeleteBuffers(1, &indexBufferID0);
//	glDeleteTextures(1, &textureID);
//}



// Test the LF2530 hardware extension technique
//void TestVertexBufferOES(int iBillboards, const int iBillboardSize=32, const int iTextureSize=512)
//{
//#ifndef EMULATION
//#ifndef LF1000
//	GLuint textureID = createTexture(iTextureSize);
//	
//	CKernelMPI kernel;
//	GLfixed *vertexTextureArray = new GLfixed[iBillboards * 4 * 8];
//	createIndexedVertexTextureArrayx(vertexTextureArray, iBillboards, iBillboardSize, iTextureSize);
//	
//	GLuint vertexTextureBufferID;
//	glEnableClientState(GL_HARDWAREBUFFER_ARRAY_OES);
//	glGenHWBuffersOES(1, &vertexTextureBufferID);
//	glBindHWBufferOES(GL_ARRAY_BUFFER, vertexTextureBufferID);
//	// TODO: why is this times 4?  Do the 0 and 1 not count?
//	glHWBufferDataOES(GL_ARRAY_BUFFER, iBillboards * 4 * sizeof(GLfixed), vertexTextureArray, GL_HWBUFFER_XVT_OES);
//	// unbind the VBO to reuse in the future
//	glBindHWBufferOES(GL_ARRAY_BUFFER, 0);
//	delete[] vertexTextureArray;
//		
//	int iTotalIndices = iBillboards * 6;
//	GLushort *indexArray0 = new GLushort[iTotalIndices];
//	createIndexArray(indexArray0, iBillboards);
//
//	GLuint indexBufferID0;
//	glGenHWBuffersOES(1, &indexBufferID0);
//	glBindHWBufferOES(GL_ELEMENT_ARRAY_BUFFER, indexBufferID0);
//	// TODO: is GL_UNSIGNED_SHORT a valid value for the last parameter here?
//	glHWBufferDataOES(GL_ELEMENT_ARRAY_BUFFER, iTotalIndices*sizeof(GLushort), indexArray0, GL_UNSIGNED_SHORT);
//	// unbind the VBO to reuse in the future
//	glBindHWBufferOES(GL_ELEMENT_ARRAY_BUFFER, 0);
//	delete[] indexArray0;
//	
//	int frames = NUMBER_FRAMES;
//	U64 startTime = kernel.GetElapsedTimeAsMSecs();
//	while(frames--)
//	{
//		gl_clearBuffer();
//		glDrawElements(GL_TRIANGLES, iTotalIndices, GL_UNSIGNED_SHORT, 0);
//		gl_swapBuffer();
//	}
//	U64 endTime = kernel.GetElapsedTimeAsMSecs();
//	U64 renderTime = endTime - startTime;
//	printf("VertexBufferOES size=%d billboards=%d fps=%f\n", iBillboardSize, iBillboards, (double)NUMBER_FRAMES * 1000 / renderTime);
//	// cleanup
//	glDeleteHWBuffersOES(1, &vertexTextureBufferID);
//	glDeleteHWBuffersOES(1, &indexBufferID0);
//	glDeleteTextures(1, &textureID);
//#endif	// not LF1000
//#endif	// not EMULATION - LF2530
//}



void test_VertexBuffer(int maxBillboards, int stride)
{
#if SUPPORT_LIB
	
	gl_disableAll();
	gl_enableDepthBuffer();
	gl_enableBlending();
	gl_enableAlpha();
	glEnable(GL_TEXTURE_2D);
	if(gl_checkError())
	{
		printf("Error:  An error occured in test_VertexBuffer after disabling and enabling features.\n");
	}
	int stride_adjusted = stride / NUM_TEXTURE_SHEETS;
	int maxBillboards_adjusted = maxBillboards / NUM_TEXTURE_SHEETS;
	for(int i = stride_adjusted; i <= maxBillboards_adjusted; i += stride_adjusted)
	{
		TestVertexBuffer(i, 32, 128);
		//TestVertexBuffer(i, 20);
		//TestVertexBuffer(i, 16);
		//TestVertexBuffer(i, 10);
		//TestVertexBuffer(i, 8);
		
		// TODO: figure out why this does not work - gives a seg fault on the LF1000
		//TestVertexBufferIndexed(i, 32);
		
		//TestSingleVertexBuffer(i, 32);
		
#ifndef EMULATION
#	ifndef LF1000	// LF2530
		//TestVertexBufferOES(i, 32);
		//TestVertexBufferOES(i, 20);
		//TestVertexBufferOES(i, 16);
		//TestVertexBufferOES(i, 10);
#	endif
#endif
		if(gl_checkError())
		{
			printf("Error:  An error occured in test_VertexBuffer after running tests.\n");
		}
	}

#endif
}


