#undef	lf_linux
#define	__int64	long long


#include "benchmark_utils.h"


//#include <BrioOpenGLConfig.h>
#include <GLES/gl.h>
#include <GLES/egl.h>

//#include "Fixed.h"
#include "render.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#define tFixed(x)		FixedFromFloat(x)
#define ONE_X			ONE
#define U8				unsigned char
#define U16				unsigned short
#define U32				unsigned long	


#define SUPPORT_LIB 1
//LF_USING_BRIO_NAMESPACE()

//BrioOpenGLConfig *pConfig;
extern EGLDisplay glesDisplay;  // EGL display
extern EGLSurface glesSurface;	 // EGL rendering surface
extern EGLContext glesContext;	 // EGL rendering context

NativeWindowType hNativeWnd; // A handle to the window we will create
GLboolean drawInOrtho;

U32	GetElapsedTimeAsMSecs()
{
    timeval time;

	gettimeofday( &time, NULL );
	
	return ( time.tv_sec * 1000 + time.tv_usec / 1000);
}

void gl_setup()
{
#if SUPPORT_LIB
	
	//pConfig = new BrioOpenGLConfig();
	hNativeWnd = &glesDisplay; 
	InitOGLES();
	
	// disable everything by default
	gl_disableAll();
	// now selectively enable
	glShadeModel(GL_SMOOTH);
	gl_enableDepthBuffer();
	gl_enableBlending();
	gl_enableAlpha();
	
	// texture
	glEnable(GL_TEXTURE_2D);
	
	//glEnableClientState(GL_VERTEX_ARRAY);//Always on
	//glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glClearColorx((GLfixed) 0, (GLfixed) 0, (GLfixed) 0, (GLfixed) 0);
	
	// Set the orthographic perspective to match the screen size.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrthox(0, tFixed(320),
			0, tFixed(240),
			tFixed(-10), tFixed(1));
	glMatrixMode(GL_MODELVIEW);
	
	#ifndef EMULATION
		#ifndef LF1000
		// only enable color key transparency on the 2530 hardware
		//glTexColorKeyOES(true, 63519);
		glTexColorKeyOES(true, -2017);	// 63519 - (2**16)
		#endif
	#endif

#endif

}

void gl_teardown()
{
	//delete pConfig;
	Clean();
}

void gl_clearBuffer()
{
#if SUPPORT_LIB

#ifdef ENABLE_DEPTH_BUFFER
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
#else
	glClear(GL_COLOR_BUFFER_BIT);
#endif

#endif

}

bool gl_swapBuffer()
{
#if SUPPORT_LIB	
	return eglSwapBuffers(glesDisplay, glesSurface); //(pConfig->eglDisplay, pConfig->eglSurface);
#endif
}

void gl_disableAll()
{

#if SUPPORT_LIB
	
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_BLEND);
	//glDisable(GL_CLIP_PLANE0);	// not supported
	//glDisable(GL_CLIP_PLANE1);
	//glDisable(GL_CLIP_PLANE2);
	//glDisable(GL_CLIP_PLANE3);
	//glDisable(GL_CLIP_PLANE4);
	//glDisable(GL_CLIP_PLANE5);
	//glDisable(GL_COLOR_LOGIC_OP);	// not supported
	glDisable(GL_COLOR_MATERIAL);
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);
	// glDisable(GL_DITHER);	// this is true by default
	glDisable(GL_FOG);
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	//glDisable(GL_LIGHT2);
	//glDisable(GL_LIGHT3);
	//glDisable(GL_LIGHT4);
	//glDisable(GL_LIGHT5);
	//glDisable(GL_LIGHT6);
	//glDisable(GL_LIGHT7);
	glDisable(GL_LIGHTING);
	glDisable(GL_LINE_SMOOTH);
	glDisable(GL_MATRIX_PALETTE_OES);
	// glDisable(GL_MULTISAMPLE);	// not supported
	//glDisable(GL_NORMALIZE);	// not supported
	glDisable(GL_POINT_SMOOTH);
	glDisable(GL_POINT_SPRITE_OES);
	glDisable(GL_POLYGON_OFFSET_FILL);
	//glDisable(GL_RESCALE_NORMAL);	// not supported
	//glDisable(GL_SAMPLE_ALPHA_TO_COVERAGE);	// not supported
	//glDisable(GL_SAMPLE_ALPHA_TO_ONE);	// not supported
	//glDisable(GL_SAMPLE_COVERAGE);	// not supported
	glDisable(GL_SCISSOR_TEST);
	//glDisable(GL_STENCIL_TEST);	// not supported
	glDisable(GL_TEXTURE_2D);
	// check error state
	if(gl_checkError())
	{
		printf("Error:  An error occured in gl_disableAll()\n");
	}

#endif
}

void gl_enableBlending()
{
#if SUPPORT_LIB

#ifdef ENABLE_BLENDING
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
#endif

#endif
}

void gl_enableDepthBuffer()
{
#if SUPPORT_LIB

#ifdef ENABLE_DEPTH_BUFFER
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
#endif

#endif
}
void gl_enableAlpha()
{
#if SUPPORT_LIB

#ifdef ENABLE_ALPHA
	glEnable(GL_ALPHA_TEST);
	glAlphaFuncx(GL_GREATER, 0);
#endif

#endif
}
bool gl_checkError()
{
#if SUPPORT_LIB	
	GLenum error = glGetError();
#else
    int error=0;
#endif

	switch (error)
	{
	case GL_NO_ERROR:
		return false;
		break;
	case GL_INVALID_ENUM:
		printf("GL_INVALID_ENUM\n");
		return true;
		break;
	case GL_INVALID_VALUE:
		printf("GL_INVALID_VALUE\n");
		return true;
		break;
	case GL_INVALID_OPERATION:
		printf("GL_INVALID_OPERATION\n");
		return true;
	case GL_STACK_OVERFLOW:
		printf("GL_STACK_OVERFLOW\n");
		return true;
		break;
	case GL_STACK_UNDERFLOW:
		printf("GL_STACK_UNDERFLOW\n");
		return true;
		break;
	case GL_OUT_OF_MEMORY:
		printf("GL_OUT_OF_MEMORY\n");
		return true;
		break;
	default:
		printf("Unknown OpenGL Error\n");
		return true;
		break;
	}
}

void print_state()
{
	printf("blending: ");
	if(ENABLE_BLENDING)			printf("enabled\n");
	else						printf("disabled\n");
	
	printf("depthbuffer: ");
	if(ENABLE_DEPTH_BUFFER)		printf("enabled\n");
	else						printf("disabled\n");
	
	printf("alpha: ");
	if(ENABLE_ALPHA)			printf("enabled\n");
	else						printf("disabled\n");
	
	printf("zmode: ");
	if(USE_ZMODE==tZModeFrontToBack)		printf("front-to-back\n");
	else if(USE_ZMODE==tZModeBackToFront)	printf("back-to-front\n");
	else if(USE_ZMODE==tZModeRandom)		printf("random\n");
	else if(USE_ZMODE==tZModeAllZero)		printf("all-zero\n");
	else									printf("unknown\n");
	
	printf("pixelformat: ");
	if(USE_PIXEL_FORMAT==GL_UNSIGNED_SHORT_4_4_4_4)			printf("RGBA4444\n");
	else if(USE_PIXEL_FORMAT==GL_UNSIGNED_SHORT_5_5_5_1)	printf("RGBA5551\n");
	else if(USE_PIXEL_FORMAT==GL_UNSIGNED_SHORT_5_6_5)		printf("RGB565\n");
	else if(USE_PIXEL_FORMAT==GL_UNSIGNED_BYTE)				printf("RGBA8888\n");
	else													printf("unknown\n");
	
	printf("textures: %d\n", NUM_TEXTURE_SHEETS);
}

void print_test_start(const char * sTestName)
{
	printf("starting %s", sTestName);
	
	printf(" depthbuffer=");
	if(ENABLE_DEPTH_BUFFER)		printf("enabled");
	else						printf("disabled");
	
	printf(" zmode=");
	if(USE_ZMODE==tZModeFrontToBack)		printf("front-to-back");
	else if(USE_ZMODE==tZModeBackToFront)	printf("back-to-front");
	else if(USE_ZMODE==tZModeRandom)		printf("random");
	else if(USE_ZMODE==tZModeAllZero)		printf("all-zero");
	else									printf("unknown");

	printf("\n");
}

void print_test_results(const char * sTestName, int iBillboardSize, int iBillboards, double fps)
{
	printf("%s size=%d billboards=%d fps=%f", sTestName, iBillboardSize, iBillboards, fps);

	printf(" depthbuffer=");
	if(ENABLE_DEPTH_BUFFER)		printf("enabled");
	else						printf("disabled");
	
	printf(" zmode=");
	if(USE_ZMODE==tZModeFrontToBack)		printf("front-to-back");
	else if(USE_ZMODE==tZModeBackToFront)	printf("back-to-front");
	else if(USE_ZMODE==tZModeRandom)		printf("random");
	else if(USE_ZMODE==tZModeAllZero)		printf("all-zero");
	else									printf("unknown");

	printf(" textures=%d", NUM_TEXTURE_SHEETS);
	
	printf(" redraw=%d", REWRITE_VERTEX_ARRAY_EACH_FRAME);
	printf("\n");

}

/*
 * tPixelFormat
 * 		R4G4B4A4	GL_UNSIGNED_SHORT_4_4_4_4
 * 		R5G6B5		GL_UNSIGNED_SHORT_5_6_5
 * 		R5G5B5A1	GL_UNSIGNED_SHORT_5_5_5_1
 * 		R8G8B8A8	GL_UNSIGNED_BYTE
 */
GLuint createTexture(const int iTextureSize, unsigned int tPixelFormat)
{
	GLuint textureID;
#if SUPPORT_LIB
	glGenTextures(1, &textureID);				// generate texture name
	glBindTexture(GL_TEXTURE_2D, textureID);	// bind the texture
	
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );	
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	
	U16* textureU16 = static_cast<U16*>(malloc(iTextureSize * iTextureSize * sizeof(U16)));
	U32* textureU32 = static_cast<U32*>(malloc(iTextureSize * iTextureSize * sizeof(U32)));
	
	U8 r, g, b, a;
	switch (tPixelFormat)
	{
		case GL_UNSIGNED_SHORT_4_4_4_4:
			for (int x = 0; x<iTextureSize; x++)
			{
				for (int y=0; y<iTextureSize; y++)
				{
					int index = x + y*iTextureSize;
					r = (x/(iTextureSize/16)) % 16;
					//g = ((2*iTextureSize-y-x)/(iTextureSize/16)) % 16;
					g = rand() % 16;
					b = ((iTextureSize-y)/(iTextureSize/16)) % 16;
					if (x%8==0 or y%8==0)
					{
						a = 0;
					}
					else
					{
						a = 15;
					}
					textureU16[index] = (r<<12)+(g<<8)+(b<<4)+a;
				}
			}
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 
					iTextureSize, iTextureSize, 0, 
					GL_RGBA, GL_UNSIGNED_SHORT_4_4_4_4, textureU16);
			break;
		case GL_UNSIGNED_SHORT_5_6_5:
			for (int x = 0; x<iTextureSize; x++)
			{
				for (int y=0; y<iTextureSize; y++)
				{
					int index = x + y*iTextureSize;
					r = (x/(iTextureSize/32)) % 32;
					//g = ((2*iTextureSize-y-x)/(iTextureSize/64)) % 64;
					g = rand() % 64;
					b = ((iTextureSize-y)/(iTextureSize/32)) % 32;
					textureU16[index] = (r<<11)+(g<<5)+b;
				}
			}
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 
					iTextureSize, iTextureSize, 0, 
					GL_RGB, GL_UNSIGNED_SHORT_5_6_5, textureU16);
			break;
		case GL_UNSIGNED_SHORT_5_5_5_1:
			for (int x = 0; x<iTextureSize; x++)
			{
				for (int y=0; y<iTextureSize; y++)
				{
					int index = x + y*iTextureSize;
					r = (x/(iTextureSize/32)) % 32;
					//r = (rand()>>8) % 32;
					//g = ((2*iTextureSize-y-x)/(iTextureSize/32)) % 32;
					g = (rand()>>8) % 32;
					b = ((iTextureSize-y)/(iTextureSize/32)) % 32;
					if (x%8==0 or y%8==0)
					{
						a = 0;
					}
					else
					{
						a = 1;
					}
					textureU16[index] = (r<<11)+(g<<6)+(b<<1)+a;
				}
			}
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 
					iTextureSize, iTextureSize, 0, 
					GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1, textureU16);
			break;
		case GL_UNSIGNED_BYTE:
			for (int x = 0; x<iTextureSize; x++)
			{
				for (int y=0; y<iTextureSize; y++)
				{
					int index = x + y*iTextureSize;
					r = (x/(iTextureSize/256)) % 256;
					//g = ((2*iTextureSize-y-x)/(iTextureSize/256)) % 256;
					g = rand() % 256;
					b = ((iTextureSize-y)/(iTextureSize/256)) % 256;
					if (x%8==0 or y%8==0)
					{
						a = 0;
					}
					else
					{
						a = 255;
					}
					textureU32[index] = (r<<24)+(g<<16)+(b<<8)+a;
				}
			}
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 
					iTextureSize, iTextureSize, 0, 
					GL_RGBA, GL_UNSIGNED_BYTE, textureU32);
			break;
		default:
			// TODO: error message
			// garbage in, garbage out
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, iTextureSize, iTextureSize, 0, GL_RGBA, GL_UNSIGNED_BYTE, textureU32);
			break;
	}
	
	
	// debug
//	for (int x = 0; x<iTextureSize; x++)
//	{
//		for (int y=0; y<iTextureSize; y++)
//		{
//			int index = x + y*iTextureSize;
//			printf("%d ", textureU16[index]);
//		}
//		printf("\n");
//	}
//	printf("\n\n");
	
	delete[] textureU16;
	delete[] textureU32;

#endif
	return textureID;
}

// TODO: DO'B - This needs to ensure that the value it returns is within the range of the viewable clip plane.
GLfixed getNextZ(GLfixed z, const tZMode zMode)
{
	switch(zMode)
	{
	case tZModeFrontToBack:
		//z--;
		z -= 1;
		break;
	case tZModeBackToFront:
		//z++;
		z += 1;
		break;
	case tZModeRandom:
		z = tFixed(1) + (rand()%65536);	// cheap way to make a fixed between 1 and 2
		break;
	case tZModeAllZero:
	default:
		z = 0;
		break;
	}
	return z;
}

// iCoordsPerVertex is 2, 3, or 4
void createIndexedVertexArrayx(GLfixed* vertexArray, const int iBillboards, 
		const int iBillboardSize, const int iCoordsPerVertex, const tZMode zMode)
{
	const int iVerticesPerBillboard = 4;		// 4 if indexed, 6 otherwise
	const GLfixed billboardSize = tFixed(iBillboardSize);
	//const int iBillboardCols = SCREEN_WIDTH / iBillboardSize;
	//const int iBillboardRows = SCREEN_HEIGHT / iBillboardSize;
	GLfixed z = tFixed(4);
	for (int j = 0; j < iBillboards; ++j)
	{
		int i = j*iVerticesPerBillboard*iCoordsPerVertex;
		//GLfixed left = tFixed((j % iBillboardCols) * iBillboardSize);
		//GLfixed bottom = tFixed(((j/iBillboardCols) % iBillboardRows) * iBillboardSize);
		GLfixed left = tFixed(rand() % (SCREEN_WIDTH-iBillboardSize));
		GLfixed bottom = tFixed(rand() % (SCREEN_HEIGHT-iBillboardSize));
		z = getNextZ(z, zMode);
		
		// The four sides of the rectangle
		// C      D
		//  |----|
		//  |    |
		//  |----|
		// A      B
		
		// A
		vertexArray[i++] = left;
		vertexArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// B
		vertexArray[i++] = left + billboardSize;
		vertexArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// C
		vertexArray[i++] = left;
		vertexArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// D
		vertexArray[i++] = left + billboardSize;
		vertexArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
	}
}

// iCoordsPerVertex is 2, 3, or 4
// TODO:  This could be made a special case of the above function
void createVertexArrayx(GLfixed* vertexArray, const int iBillboards, 
		const int iBillboardSize, const int iCoordsPerVertex, const tZMode zMode)
{
	const int iVerticesPerBillboard = 6;		// 4 if indexed, 6 otherwise
	const GLfixed billboardSize = tFixed(iBillboardSize);
	//const int iBillboardCols = SCREEN_WIDTH / iBillboardSize;
	//const int iBillboardRows = SCREEN_HEIGHT / iBillboardSize;
	GLfixed z = tFixed(4);
	for (int j = 0; j < iBillboards; ++j)
	{
		int i = j*iVerticesPerBillboard*iCoordsPerVertex;
		//GLfixed left = tFixed((j % iBillboardCols) * iBillboardSize);
		//GLfixed bottom = tFixed(((j/iBillboardCols) % iBillboardRows) * iBillboardSize);
		GLfixed left = tFixed(rand() % (SCREEN_WIDTH-iBillboardSize));
		GLfixed bottom = tFixed(rand() % (SCREEN_HEIGHT-iBillboardSize));
		z = getNextZ(z, zMode);
		
		// The four sides of the rectangle
		// C      D
		//  |----|
		//  |    |
		//  |----|
		// A      B
		
		// A
		vertexArray[i++] = left;
		vertexArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// B
		vertexArray[i++] = left + billboardSize;
		vertexArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// C
		vertexArray[i++] = left;
		vertexArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// B
		vertexArray[i++] = left + billboardSize;
		vertexArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// D
		vertexArray[i++] = left + billboardSize;
		vertexArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
		// C
		vertexArray[i++] = left;
		vertexArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexArray[i++] = ONE_X;
	}
}



void createIndexedTextureCoordArrayx(GLfixed* textureArray, const int iBillboards, 
		const int iBillboardSize, const int iTextureSize, const int iCoordsPerVertex)
{
	const int iVerticesPerBillboard = 4;		// 4 if indexed, 6 otherwise
	const int textureWidth = iTextureSize;
	const int textureHeight = iTextureSize;
	// how many rows is the texture divided into
	const int textureCols = textureWidth/iBillboardSize;
	const int textureRows = textureHeight/iBillboardSize;
	// what fraction of the texture to use
	const GLfixed texture_size = tFixed(1.0/(float)textureCols);
	for (int j = 0; j < iBillboards; ++j)
	{
		int i = j*iVerticesPerBillboard*iCoordsPerVertex;
		GLfixed textcoord_left = (j%textureCols)*texture_size;
		GLfixed textcoord_bottom = ((j/textureCols)%textureRows) * texture_size;
		
		//printf("%d, %d, %d, %d\n", textcoord_left, textcoord_bottom, textcoord_left + texture_size, textcoord_bottom + texture_size);
		// The four sides of the rectangle
		// C      D
		//  |----|
		//  |    |
		//  |----|
		// A      B

		// A
		textureArray[i++] = textcoord_left;
		textureArray[i++] = textcoord_bottom;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// B
		textureArray[i++] = textcoord_left + texture_size;
		textureArray[i++] = textcoord_bottom;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// C
		textureArray[i++] = textcoord_left;
		textureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// D
		textureArray[i++] = textcoord_left + texture_size;
		textureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
	}
}

// TODO:  This could be made a special case of the above function
void createTextureCoordArrayx(GLfixed* textureArray, const int iBillboards, 
		const int iBillboardSize, const int iTextureSize, const int iCoordsPerVertex)
{
	const int iVerticesPerBillboard = 6;		// 4 if indexed, 6 otherwise
	const int textureWidth = iTextureSize;
	const int textureHeight = iTextureSize;
	// how many rows is the texture divided into
	const int textureCols = textureWidth/iBillboardSize;
	const int textureRows = textureHeight/iBillboardSize;
	// what fraction of the texture to use
	const GLfixed texture_size = tFixed(1.0/(float)textureCols);
	for (int j = 0; j < iBillboards; ++j)
	{
		int i = j*iVerticesPerBillboard*iCoordsPerVertex;
		GLfixed textcoord_left = (j%textureCols)*texture_size;
		GLfixed textcoord_bottom = ((j/textureCols)%textureRows) * texture_size;
		
		//printf("%d, %d, %d, %d\n", textcoord_left, textcoord_bottom, textcoord_left + texture_size, textcoord_bottom + texture_size);
		// The four sides of the rectangle
		// C      D
		//  |----|
		//  |    |
		//  |----|
		// A      B
		
		// A
		textureArray[i++] = textcoord_left;
		textureArray[i++] = textcoord_bottom;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// B
		textureArray[i++] = textcoord_left + texture_size;
		textureArray[i++] = textcoord_bottom;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// C
		textureArray[i++] = textcoord_left;
		textureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// B
		textureArray[i++] = textcoord_left + texture_size;
		textureArray[i++] = textcoord_bottom;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// D
		textureArray[i++] = textcoord_left + texture_size;
		textureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
		// C
		textureArray[i++] = textcoord_left;
		textureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerVertex>=3)	textureArray[i++] = 0;
		if (iCoordsPerVertex>=4)	textureArray[i++] = ONE_X;
	}
}

void createIndexArray(GLushort* indexArray, const int iBillboards)
{
	for(int b=0; b<iBillboards; ++b)
	{
		int i = 6*b;
		int j = 4*b;
		indexArray[i++] = j;
		indexArray[i++] = j + 1;
		indexArray[i++] = j + 2;
		indexArray[i++] = j + 1;
		indexArray[i++] = j + 3;
		indexArray[i++] = j + 2;
	}
}

void createVertexTextureArrayx(GLfixed* vertexTextureArray,
		const int iBillboards,
		const int iBillboardSize,
		const int iTextureSize, 
		const int iCoordsPerVertex,
		const int iCoordsPerTextureVertex,
		const tZMode zMode)
{
	const int iVerticesPerBillboard = 6;		// 4 if indexed, 6 otherwise
	const GLfixed billboardSize = tFixed(iBillboardSize);
	//const int iBillboardCols = SCREEN_WIDTH / iBillboardSize;
	//const int iBillboardRows = SCREEN_HEIGHT / iBillboardSize;
	const int textureWidth = iTextureSize;
	const int textureHeight = iTextureSize;
	// how many rows is the texture divided into
	const int textureCols = textureWidth/iBillboardSize;
	const int textureRows = textureHeight/iBillboardSize;
	// what fraction of the texture to use
	const GLfixed texture_size = tFixed(1.0/(float)textureCols);
	GLfixed z = tFixed(4);
	for (int j = 0; j < iBillboards; ++j)
	{
		z = getNextZ(z, zMode);
		
		int i = j*iVerticesPerBillboard*(iCoordsPerVertex+iCoordsPerTextureVertex);
		//GLfixed left = tFixed((j % iBillboardCols) * iBillboardSize);
		//GLfixed bottom = tFixed(((j/iBillboardCols) % iBillboardRows) * iBillboardSize);
		GLfixed left = tFixed(rand() % (SCREEN_WIDTH-iBillboardSize));
		GLfixed bottom = tFixed(rand() % (SCREEN_HEIGHT-iBillboardSize));
		GLfixed textcoord_left = (j%textureCols)*texture_size;
		GLfixed textcoord_bottom = ((j/textureCols)%textureRows) * texture_size;
		// The four sides of the rectangle
		// C      D
		//  |----|
		//  |    |
		//  |----|
		// A      B
		
		// A
		vertexTextureArray[i++] = left;
		vertexTextureArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left;
		vertexTextureArray[i++] = textcoord_bottom;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// B
		vertexTextureArray[i++] = left + billboardSize;
		vertexTextureArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left + texture_size;
		vertexTextureArray[i++] = textcoord_bottom;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// C
		vertexTextureArray[i++] = left;
		vertexTextureArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left;
		vertexTextureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;

		// B
		vertexTextureArray[i++] = left + billboardSize;
		vertexTextureArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left + texture_size;
		vertexTextureArray[i++] = textcoord_bottom;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// D
		vertexTextureArray[i++] = left + billboardSize;
		vertexTextureArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left + texture_size;
		vertexTextureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// C
		vertexTextureArray[i++] = left;
		vertexTextureArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
				
		vertexTextureArray[i++] = textcoord_left;
		vertexTextureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
	}
}

void createIndexedVertexTextureArrayx(GLfixed* vertexTextureArray,
		const int iBillboards,
		const int iBillboardSize,
		const int iTextureSize, 
		const int iCoordsPerVertex,
		const int iCoordsPerTextureVertex,
		const tZMode zMode)
{
	const int iVerticesPerBillboard = 4;		// 4 if indexed, 6 otherwise
	const GLfixed billboardSize = tFixed(iBillboardSize);
	//const int iBillboardCols = SCREEN_WIDTH / iBillboardSize;
	//const int iBillboardRows = SCREEN_HEIGHT / iBillboardSize;
	const int textureWidth = iTextureSize;
	const int textureHeight = iTextureSize;
	// how many rows is the texture divided into
	const int textureCols = textureWidth/iBillboardSize;
	const int textureRows = textureHeight/iBillboardSize;
	// what fraction of the texture to use
	const GLfixed texture_size = tFixed(1.0/(float)textureCols);
	GLfixed z = tFixed(4);
	for (int j = 0; j < iBillboards; ++j)
	{
		z = getNextZ(z, zMode);
		
		int i = j*iVerticesPerBillboard*(iCoordsPerVertex+iCoordsPerTextureVertex);
		//GLfixed left = tFixed((j % iBillboardCols) * iBillboardSize);
		//GLfixed bottom = tFixed(((j/iBillboardCols) % iBillboardRows) * iBillboardSize);
		GLfixed left = tFixed(rand() % (SCREEN_WIDTH-iBillboardSize));
		GLfixed bottom = tFixed(rand() % (SCREEN_HEIGHT-iBillboardSize));
		GLfixed textcoord_left = (j%textureCols)*texture_size;
		GLfixed textcoord_bottom = ((j/textureCols)%textureRows) * texture_size;
		// The four sides of the rectangle
		// C      D
		//  |----|
		//  |    |
		//  |----|
		// A      B
		
		// A
		vertexTextureArray[i++] = left;
		vertexTextureArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left;
		vertexTextureArray[i++] = textcoord_bottom;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// B
		vertexTextureArray[i++] = left + billboardSize;
		vertexTextureArray[i++] = bottom;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left + texture_size;
		vertexTextureArray[i++] = textcoord_bottom;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// C
		vertexTextureArray[i++] = left;
		vertexTextureArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left;
		vertexTextureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		// D
		vertexTextureArray[i++] = left + billboardSize;
		vertexTextureArray[i++] = bottom + billboardSize;
		if (iCoordsPerVertex>=3)	vertexTextureArray[i++] = z;
		if (iCoordsPerVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
		vertexTextureArray[i++] = textcoord_left + texture_size;
		vertexTextureArray[i++] = textcoord_bottom + texture_size;
		if (iCoordsPerTextureVertex>=3)	vertexTextureArray[i++] = 0;
		if (iCoordsPerTextureVertex>=4)	vertexTextureArray[i++] = ONE_X;
		
	}
}

