#ifndef _UTILS_H_
#define _UTILS_H_

#include <GLES/gl.h>
#include <GLES/egl.h>
#ifndef EMULATION
#include <GLES/glext.h>
#endif

#define SCREEN_WIDTH	320
#define SCREEN_HEIGHT	240
#define NUMBER_FRAMES	128

typedef enum tZMode
{
	tZModeFrontToBack,
	tZModeBackToFront,
	tZModeAllZero,
	tZModeRandom
} tZMode;

//----------------------------------------------------------------------------------
// Change these values to enable and disable features - blending, alpha, depth buffer, etc.
//

#define USE_ZMODE	tZModeFrontToBack
//#define USE_ZMODE	tZModeBackToFront
//#define USE_ZMODE	tZModeAllZero
//#define USE_ZMODE	tZModeRandom

//#define ENABLE_DEPTH_BUFFER		0
#define ENABLE_DEPTH_BUFFER		1

#define NUM_TEXTURE_SHEETS		20

#define REWRITE_VERTEX_ARRAY_EACH_FRAME 	1	// 0 or 1

#define COORDS_PER_VERTEX			4
#define COORDS_PER_TEXTURE_VERTEX	4


// These can be changed but don't seem to have much effect

//#define USE_PIXEL_FORMAT	GL_UNSIGNED_SHORT_4_4_4_4
//#define USE_PIXEL_FORMAT	GL_UNSIGNED_SHORT_5_6_5
#define USE_PIXEL_FORMAT	GL_UNSIGNED_SHORT_5_5_5_1
//#define USE_PIXEL_FORMAT	GL_UNSIGNED_BYTE

//#define ENABLE_BLENDING	0
#define ENABLE_BLENDING	0

//#define ENABLE_ALPHA	0
#define ENABLE_ALPHA	1

//
//----------------------------------------------------------------------------------


void gl_setup();
void gl_teardown();
void gl_clearBuffer();
bool gl_swapBuffer();
void gl_disableAll();
void gl_enableBlending();
void gl_enableDepthBuffer();
void gl_enableAlpha();
bool gl_checkError();
void print_state();
void print_test_start(const char * sTestName);
void print_test_results(const char * sTestName, int iBillboardSize, int iBillboards, double fps);

GLuint createTexture(const int iTextureSize, unsigned int tPixelFormat=GL_UNSIGNED_BYTE);

/**
 * Creates a vertex array that is designed to be used with an index array.
 * That is, it has four points per billboard.
 */
void createIndexedVertexArrayx(GLfixed* vertexArray, 
		const int iBillboards, 
		const int iBillboardSize, 
		const int iCoordsPerVertex, 
		const tZMode zMode=tZModeAllZero);
/**
 * Creates a vertex array with six points per billboard.
 */
void createVertexArrayx(GLfixed* vertexArray, 
		const int iBillboards, 
		const int iBillboardSize, 
		const int iCoordsPerVertex, 
		const tZMode zMode=tZModeAllZero);

/**
 * Creates a texture coordinate array that is designed to be used with an index array.
 * That is, it has four points per billboard.
 */
void createIndexedTextureCoordArrayx(GLfixed* textureArray, 
		const int iBillboards, 
		const int iBillboardSize, 
		const int iTextureSize, 
		const int iCoordsPerVertex);
/**
 * Creates a texture coordinate array with six points per billboard.
 */
void createTextureCoordArrayx(GLfixed* textureArray, 
		const int iBillboards, 
		const int iBillboardSize, 
		const int iTextureSize, 
		const int iCoordsPerVertex);

void createIndexArray(GLushort* indexArray, const int iBillboards);

void createIndexedVertexTextureArrayx(GLfixed* vertexTextureArray, 
		const int iBillboards, 
		const int iBillboardSize, 
		const int iTextureSize,
		const tZMode zMode=tZModeAllZero);
/**
 * Creates an array of vertex data and texture coord data packed in a single array.
 * This is not designed to be indexed.
 */
void createVertexTextureArrayx(GLfixed* vertexTextureArray,
		const int iBillboards,
		const int iBillboardSize,
		const int iTextureSize, 
		const int iCoordsPerVertex,
		const int iCoordsPerTextureVertex,
		const tZMode zMode=tZModeAllZero);

#endif	//  _UTILS_H_

